%code top {
/* Load config.h, and adjust to the compiler.
  We used to do it here, but each time we add a new line,
  we have to adjust all the line numbers in error messages.
  It's simpler to use a constant include to a varying file.  */
#include <testsuite.h>
}

%define api.prefix {x4_}
%locations %debug %glr-parser
%define parse.error verbose
%union
{
  int integer;
}
%{

  #include <stdio.h>

#if defined X4_LTYPE_IS_TRIVIAL && X4_LTYPE_IS_TRIVIAL
static int location_print (FILE *yyo, X4_LTYPE const * const yylocp);
# ifndef LOCATION_PRINT
#  define LOCATION_PRINT(File, Loc) location_print (File, &(Loc))
# endif
#endif

static void x4_error (const char *msg);
  static int x4_lex (void);
%}
%%
exp:
  'x' '1' { printf ("x1\n"); }
| 'x' '2' { printf ("x2\n"); }
| 'x' '3' { printf ("x3\n"); }
| 'x' '4' { printf ("x4\n"); }
| 'x' '5' { printf ("x5\n"); }
| 'x' '6' { printf ("x6\n"); }
| 'x' '7' { printf ("x7\n"); }
| 'x' '8' { printf ("x8\n"); }
| 'x' '9' { printf ("x9\n"); }
;

%%


# if defined X4_LTYPE_IS_TRIVIAL && X4_LTYPE_IS_TRIVIAL
/* Print *YYLOCP on YYO. */
__attribute__((__unused__))
static int
location_print (FILE *yyo, X4_LTYPE const * const yylocp)
{
  int res = 0;
  int end_col = 0 != yylocp->last_column ? yylocp->last_column - 1 : 0;
  if (0 <= yylocp->first_line)
    {
      res += fprintf (yyo, "%d", yylocp->first_line);
      if (0 <= yylocp->first_column)
        res += fprintf (yyo, ".%d", yylocp->first_column);
    }
  if (0 <= yylocp->last_line)
    {
      if (yylocp->first_line < yylocp->last_line)
        {
          res += fprintf (yyo, "-%d", yylocp->last_line);
          if (0 <= end_col)
            res += fprintf (yyo, ".%d", end_col);
        }
      else if (0 <= end_col && yylocp->first_column < end_col)
        res += fprintf (yyo, "-%d", end_col);
    }
  return res;
}
#endif

/* A C error reporting function.  */
static
void x4_error (const char *msg)
{
  LOCATION_PRINT (stderr, (x4_lloc));
  fprintf (stderr, ": ");
  fprintf (stderr, "%s\n", msg);
}
#include <assert.h>
static
int x4_lex (void)
{
  static char const input[] = "x4";
  static size_t toknum = 0;
  int res;

  assert (toknum < sizeof input / sizeof input[0]);
  res = input[toknum++];
  ;

  (x4_lloc).first_line = (x4_lloc).last_line = 1;
  (x4_lloc).first_column = (x4_lloc).last_column = (int)toknum;
  return res;
}
